//+------------------------------------------------------------------+
//|                    Chande Trend Index (CTI).mq4                  |
//|                                                                  |
//|                  Developed by Tushar S. Chande                   |
//|                                                                  |
//|      "Beyond Technical Analysis, 2. Edition - May 2001"          |
//|                                                                  |
//|            Coded in MQL4 by *PleiteGeier*,  August 2010          |
//+------------------------------------------------------------------+


#property indicator_separate_window
//----
#property indicator_buffers 3
#property indicator_color1  Gold
#property indicator_width1  1
#property indicator_style1  STYLE_SOLID
#property indicator_color2  Red
#property indicator_width2  1
#property indicator_style2  STYLE_DOT
#property indicator_color3  White
#property indicator_width3  1
#property indicator_style3  STYLE_SOLID


#property indicator_levelcolor EMPTY
//---- extern input
extern int              n = 21;   // Look-back period (21 Days).
extern double Trend_Level = 1.0;  // Above these level, we have no more a sideways market.

//---- buffers
double CTI[];

double Level0[], Level1[];

//---- vars
double X, S;
double SumX, SumSqrt;  
string IndiShortName;
//+--------------------------------------------------------------------------+ 
//|  Chande Trend Index (CTI)  ~   initialization function                   |
//+--------------------------------------------------------------------------+
int init()
  {
//---- indicators setting
   IndicatorDigits(Digits);
   //----
   IndicatorBuffers(3);
   //----
   SetIndexStyle(0, DRAW_LINE);
   SetIndexStyle(1, DRAW_LINE);
   SetIndexStyle(2, DRAW_LINE);
   //----
   SetIndexBuffer(0, CTI);    
   SetIndexBuffer(1, Level1); 
   SetIndexBuffer(2, Level0);
   //----
   SetIndexLabel(1,NULL);
   SetIndexLabel(2,NULL);
   //---- Levels
   SetLevelValue(1, 0.0); 
   SetLevelValue(2, 0.25); 
   SetLevelValue(3, 0.5); 
   SetLevelValue(4, 0.75); 
   SetLevelValue(5, 1.0);  
   SetLevelValue(6, 1.5); 
   SetLevelValue(7, 2.0); 
   SetLevelValue(8, 2.5);  
   SetLevelValue(9, 3.0);    
   //----
   IndiShortName = "CTI ["+Symbol()+PeriodSelect()+"] ("+n+")";
   IndicatorShortName(IndiShortName);    
//---- initialization complete
   return(0);
  }
//+---------------------------------------------------------------------------+
//| Chande Trend Index (CTI)  ~  iteration function                           |
//+---------------------------------------------------------------------------+
int start()
  {
//----+ check whether the amount of bars is enough for correct indicator calculation
   if(Bars - 1 < n) return(0);
//----+ introduction of integer variables and getting of bars already counted
   int MaxBar, limit, counted_bars = IndicatorCounted();
//---- check for possible errors
   if(counted_bars < 0) return(-1);
//---- the last counted bar must be re-counted
   if(counted_bars > 0) counted_bars--;
//---- determining of the oldest bar number, starting from which all bars will be re-counted
   MaxBar = Bars - 1 - n;
//---- determining of the oldest bar number, starting from which only new bars will be re-counted
   limit = Bars - 1 - counted_bars;
//---- initialization of null
   if(limit > MaxBar)
     {
      for(int bar = Bars - 1; bar >= MaxBar; bar--) CTI[bar] = 0.0;
      limit = MaxBar; 
     }
//+--- +==========================================+
//+--- Basic loop to calculate the indicator
   for(bar = limit; bar >= 0; bar--)
     {  
      SumX = 0.0;
      
      for(int i = 0; i < n; i++) SumX += MathLog( Close[bar+i] / Close[bar+i+1] );
      
      X = SumX / n;
      
      //----
   
      SumSqrt = 0.0; 
      
      for(i = 0; i < n; i++) SumSqrt += MathPow( MathLog( Close[bar+i] / Close[bar+i+1]) - X,2  );
      
      S = MathSqrt(SumSqrt / n);
      
      //----
   
      CTI[bar] = MathAbs( MathLog( Close[bar] / Close[bar+n] ) ) / ( S * MathSqrt(n) ); 
      
      //---- No part of the formula - only for level drawing!
      Level1[bar] = Trend_Level;
      Level0[bar] = 0.0;
      //----
      Make_Front();
     } 
   //+--- terminate the basic loop
   return(0);
  }
//+-----------------------------------------------------------------+ 

//*************************************************\\
//---** Function: Make Period to Time-String. **---\\
//*************************************************\\
string PeriodSelect()
  {
  switch(Period())
     {
     case  PERIOD_M1  : return(",M1"); 
     case  PERIOD_M5  : return(",M5"); 
     case  PERIOD_M15 : return(",M15"); 
     case  PERIOD_M30 : return(",M30");
     case  PERIOD_H1  : return(",H1"); 
     case  PERIOD_H4  : return(",H4");
     case  PERIOD_D1  : return(",D1"); 
     case  PERIOD_W1  : return(",W1"); 
     case  PERIOD_MN1 : return(",MN1");
     default          : return("[ERROR] ...unknown Timeframe...");
     }
  }

//****************************************************\\
//---** Function: Draw a V-Line on the Zero-Bar. **---\\
//****************************************************\\ 
void Make_Front() 
  {
  if(ObjectFind("_Front") == -1) ObjectCreate("_Front",OBJ_VLINE,WindowFind(IndiShortName),Time[0],0,0,0);
  else
    {
    ObjectSet("_Front",OBJPROP_TIME1,Time[0]);
	 }
  ObjectSet("_Front",OBJPROP_COLOR,Aqua);
  ObjectSet("_Front",OBJPROP_STYLE,STYLE_DOT);
  ObjectSet("_Front",OBJPROP_BACK,true);
  }  